require "soar_sc/rack/router/builder_syntax"
require "soar_sc/rack/router/errors"
require "soar_sc/rack/router/http_method"
require "soar_sc/rack/router/route"
require "soar_sc/rack/router/version"

require "uri"

module SoarSc
  module Rack
    class Router

      include BuilderSyntax

      EMPTY_STRING = ""

      def initialize(app, routes = nil, &block)
        @app = app
        @routes = []
        if !routes.nil?
          raise NotImplementedError, "Route map syntax obsoleted (use builder syntax)"
        end
        instance_eval(&block) if block_given?
      end

      def call(env)
        if route = @routes.detect { |r| r.matches?(env) }
          update_path_parameters!(env, route.extract_path_parameters(env))
          route.action.call(env)
        else
          @app.call(env)
        end
      end

      private

      # Called by BuilderSyntax methods
      def add_route(route)
        if include?(route)
          raise DuplicateRouteError.for_route(route)
        else
          @routes << route
        end
      end

      def include?(route)
        @routes.detect do |other|
          route.matches?(env_for_route(other)) or other.matches?(env_for_route(route))
        end
      end

      # Creates a minimal env that would be matched by the given route.
      def env_for_route(route)
        {
          Route::REQUEST_METHOD => (route.method == HttpMethod::ANY ? HttpMethod::GET : route.method),
          Route::SCRIPT_NAME => route.path,
          Route::PATH_INFO => EMPTY_STRING
        }
      end

      def update_path_parameters!(env, params)
        unless params.empty?
          req = ::Rack::Request.new(env)
          params.each { |p, v| req.update_param(p, v) }
        end
      end

    end
  end
end
