require 'soar_sc/rack/router/http_method'
require 'soar_sc/rack/router/route_validation'

# TODO Normalize paths (compress runs of slashes)
module SoarSc
  module Rack
    class Router

      class Route

        include RouteValidation

        PARAMETER_PREFIX = ":"
        PARAMETER_VALID_REGEXP = /:[a-z_][a-z0-9_]*/
        REQUEST_METHOD = "REQUEST_METHOD"
        SCRIPT_NAME = "SCRIPT_NAME"
        PATH_INFO = "PATH_INFO"
        EMPTY_HASH = {}.freeze

        attr_reader :method, :path, :action

        def initialize(method, path, action)
          @method, @path, @action = method, path, action
          validate
          @static = components(@path).none? { |c| c.start_with?(PARAMETER_PREFIX) }
        end

        def matches?(env)
          method_matches?(env[REQUEST_METHOD]) and path_matches?(request_path(env))
        end

        def extract_path_parameters(env)
          return EMPTY_HASH if @static

          req_components = components(request_path(env))
          parameters = {}
          components(@path).each_with_index do |c, i|
            parameters[c[1..-1]] = req_components[i] if c.start_with?(PARAMETER_PREFIX)
          end
          parameters
        end

        def to_s
          "<#{@method.inspect} #{@path.inspect} #{@action.inspect}>"
        end

        private

        def method_matches?(method)
          @method == method or @method == HttpMethod::ANY
        end

        def path_matches?(path)
          if @static
            static_path_matches?(path)
          else
            parameterized_path_matches?(path)
          end
        end

        def static_path_matches?(path)
          path == @path
        end

        def parameterized_path_matches?(path)
          route_components = components(@path)
          req_components = components(path)

          return false unless route_components.size == req_components.size

          route_components.each_with_index do |c, i|
            return false unless c.start_with?(PARAMETER_PREFIX) or c == req_components[i]
          end

          return true
        end

        def request_path(env)
          env[SCRIPT_NAME] + env[PATH_INFO]
        end

        def components(path)
          path.split('/')
        end

      end

    end
  end
end

