module SoarSc
  module Rack
    class Router

      module RouteValidation

        def validate
          validate_method
          validate_path
          validate_action
        end

        def validate_method
          unless HttpMethod.valid?(@method)
            raise InvalidMethodError, "Invalid method in route #{to_s}"
          end
        end

        def validate_path
          unless @path.is_a?(String)
            raise InvalidPathError, "Invalid path in route #{to_s}"
          end
          components(@path).each do |c|
            if c.start_with?(Route::PARAMETER_PREFIX) and c !~ Route::PARAMETER_VALID_REGEXP
              raise InvalidParameterError, "Invalid path parameter in route #{to_s}"
            end
          end
        end

        def validate_action
          unless (@action.respond_to?(:arity) and @action.arity == 1) or (@action.respond_to?(:call) and @action.method(:call).arity == 1)
            raise InvalidActionError, "Invalid action in route #{to_s}"
          end
        end

      end

    end
  end
end
